
"""
Compare our clustering method and that of Ning et al. on the HIV data.
"""
import sys 
import logging
import numpy
from apgl.graph import *
from apgl.viroscopy.HIVGraphReader import HIVGraphReader
from apgl.util.PathDefaults import PathDefaults
from apgl.util.DateUtils import DateUtils
from apgl.sandbox.IterativeSpectralClustering import IterativeSpectralClustering, IncreasingSubgraphListIterator
from apgl.sandbox.NingSpectralClustering import NingSpectralClustering

numpy.random.seed(21)
numpy.seterr("raise")
logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
numpy.set_printoptions(suppress=True, linewidth=60)

#Start off with the HIV data 
hivReader = HIVGraphReader()
graph = hivReader.readHIVGraph()
fInds = hivReader.getIndicatorFeatureIndices()

#The set of edges indexed by zeros is the contact graph
#The ones indexed by 1 is the infection graph
edgeTypeIndex1 = 0
edgeTypeIndex2 = 1
sGraphContact = graph.getSparseGraph(edgeTypeIndex1)
sGraphInfect = graph.getSparseGraph(edgeTypeIndex2)
sGraphContact = sGraphContact.union(sGraphInfect)
graph = sGraphContact

#Find max component
components = graph.findConnectedComponents()
graph = graph.subgraph(list(components[0]))
logging.debug(graph)

resultsDir = PathDefaults.getOutputDir() + "cluster/"

detectionIndex = fInds["detectDate"]
vertexArray = graph.getVertexList().getVertices()
detections = vertexArray[:, detectionIndex]

startYear = 1900
daysInMonth = 30
monthStep = 3
dayList = list(range(int(numpy.min(detections)), int(numpy.max(detections)), daysInMonth*monthStep))
dayList.append(numpy.max(detections))

subgraphIndicesList = []
minGraphSize = 500

#Generate subgraph indices list 
for i in dayList:
    logging.info("Date: " + str(DateUtils.getDateStrFromDay(i, startYear)))
    subgraphIndices = numpy.nonzero(detections <= i)[0]
    if subgraphIndices.shape[0] >= minGraphSize: 
        subgraphIndicesList.append(subgraphIndices)

#subgraphIndicesList2 = []
#for i in range(5):
#    subgraphIndicesList2.append(subgraphIndicesList[i])
#subgraphIndicesList = subgraphIndicesList2

numMethods = 3
numMeasures = 2 
numGraphs = len(subgraphIndicesList)
modularities = numpy.zeros((numGraphs, numMethods))
kwayNormalisedCuts = numpy.zeros((numGraphs, numMethods))

k1 = 25
k2 = 2*k1

#Variables to choose which methods to run
runIASC = True
runExact = True
runNing = True

if runIASC or runExact:
    clusterer = IterativeSpectralClustering(k1, k2)
    clusterer.nb_iter_kmeans = 20

def recordResults(clusterList, timeList, fileName):
    iterator = IncreasingSubgraphListIterator(graph, subgraphIndicesList)
    measures = numpy.zeros((numGraphs, numMeasures))

    for i in range(numGraphs):
        W = iterator.next()
        measures[i, 0] = GraphUtils.modularity(W, clusterList[i])
        measures[i, 1] = GraphUtils.kwayNormalisedCut(W, clusterList[i])

    file = open(fileName, 'w')
    numpy.savez(file, measures, timeList)
    logging.info("Saved file as " + resultsFileName)

if runIASC:
    logging.info("Running approximate method")
    iterator = IncreasingSubgraphListIterator(graph, subgraphIndicesList)
    clusterList, timeList = clusterer.clusterFromIterator(iterator, timeIter=True)

    resultsFileName = resultsDir + "HIVResultsIASC.npz"
    recordResults(clusterList, timeList, resultsFileName)

if runExact:
    logging.info("Running exact method")
    iterator = IncreasingSubgraphListIterator(graph, subgraphIndicesList)
    clusterList, timeList = clusterer.clusterFromIterator(iterator, False, timeIter=True)

    resultsFileName = resultsDir + "HIVResultsExact.npz"
    recordResults(clusterList, timeList, resultsFileName)

if runNing:
    logging.info("Running Nings method")
    iterator = IncreasingSubgraphListIterator(graph, subgraphIndicesList)
    clusterer = NingSpectralClustering(k1)
    clusterList, timeList = clusterer.cluster(iterator, timeIter=True)

    resultsFileName = resultsDir + "HIVResultsNing.npz"
    recordResults(clusterList, timeList, resultsFileName)