
"""
A new approach to spectral clustering based on iterative eigen decomposition.
"""
import numpy
import scipy.cluster.vq as vq 
from apgl.sandbox.EigenUpdater import EigenUpdater
from apgl.data.Standardiser import Standardiser
from apgl.graph import *
from apgl.util.Parameter import Parameter 
from apgl.graph.DictGraph import DictGraph    
import itertools
import sys
import logging
import time 

class IterativeSpectralClustering(object):
    def __init__(self, k1, k2):
        """
        Intialise this object with integer k1 which is the number of clusters to
        find, and k2 which is the maximum rank of the approximation of the shift
        Laplacian. 
        """
        self.k1 = k1
        self.k2 = k2

        self.nb_iter_kmeans = 100

    def shiftLaplacian(self, W):
        """
        Get the shift laplacian matrix from W. For vertices of degree zero, the
        corresponding row/col of the Laplacian is zero with a 1 at the diagonal.
        """
        d = numpy.sum(W, 0)
        d = d + numpy.array(d==0, numpy.int)
        D = numpy.diag(d**-0.5)
        Lhat = numpy.eye(W.shape[0]) + D.dot(W).dot(D)
        return Lhat 

    def findCentroids(self, V, clusters):
        """
        Take an array of clusters and find the centroids.
        """
        k = numpy.unique(clusters).shape[0]
        centroids = numpy.zeros((k, V.shape[1]))

        for i in range(k):
            centroids[i, :] = numpy.mean(V[clusters==i, :], 0)
            
        return centroids 

    def cluster(self, graph, subgraphIndicesList, approx=True, T=10):
        """
        Find a set of clusters for the given graph and the subgraphs indexed
        using the elements of subgraphIndicesList. If approx is True then
        we use the approximate eigen-update, otherwise we compute the whole
        eigen decomposition. Each element of subgraphIndicesList must be a subset
        of the next. Furthermore the returned clustering indices correspond to the
        sorted input indices.

        The parameter T controls how frequently eigenvectors are recomputed. 

        """
        Parameter.checkClass(graph, AbstractMatrixGraph)
        Parameter.checkClass(subgraphIndicesList, list)
        clustersList = []
        W = graph.getWeightMatrix()

        subgraphIndices = numpy.array(subgraphIndicesList[0], numpy.int)
        subgraphIndices.sort()
        subW = W[:, subgraphIndices][subgraphIndices, :]
        ABBA = self.shiftLaplacian(subW)

        if approx:
            ABBALast = ABBA.copy()
            degrees = numpy.sum(subW, 0)
            n = ABBA.shape[0]

        #We want to find the largest eigenvectors
        omega, Q = numpy.linalg.eig(ABBA)
        inds = numpy.flipud(numpy.argsort(omega))

        #First normalise rows, then columns
        standardiser = Standardiser()
        V = standardiser.normaliseArray(Q[:, inds[0:self.k1]].T).T
        V = vq.whiten(V)
        #Using kmeans2 here seems to result in a high variance
        #in the quality of clustering. Therefore stick to kmeans 
        centroids, clusters = vq.kmeans(V, self.k1, iter=self.nb_iter_kmeans)
        clusters, distortion = vq.vq(V, centroids)
        centroids = centroids[numpy.unique(clusters),:]

        clustersList.append(clusters)
        if approx:
            eigenUpdaterWrapper = EigenUpdaterWrapper(self.k2)
            eigenUpdaterWrapper.storeInformation(subW, ABBA)

        for i in range(1, len(subgraphIndicesList)):
            newSubgraphIndices = numpy.setdiff1d(numpy.array(subgraphIndicesList[i], numpy.int), subgraphIndices)
            subgraphIndices = numpy.r_[subgraphIndices, newSubgraphIndices]
            subW = W[:, subgraphIndices][subgraphIndices, :]
            ABBA = self.shiftLaplacian(subW)

            # --- Eigen value decomposition ---
            if approx and i % T != 0:
                omega, Q = eigenUpdaterWrapper.approxUpdateEig(subW, ABBA, omega, Q)
            else:
                if approx and i != 0:
                    logging.info("Recomputing eigenvectors")
                omega, Q = numpy.linalg.eig(ABBA)

            if approx:
                eigenUpdaterWrapper.storeInformation(subW, ABBA)

            # --- Kmeans ---
            inds = numpy.flipud(numpy.argsort(omega))

            standardiser = Standardiser()
            V = standardiser.normaliseArray(Q[:, inds[0:self.k1]].real.T).T
            V = vq.whiten(V)
            centroids = self.findCentroids(V, clusters)
            if centroids.shape[0] < self.k1:
                nb_missing_centroids = self.k1 - centroids.shape[0]
                random_centroids = V[numpy.random.randint(0, V.shape[0], nb_missing_centroids),:]
                centroids = numpy.vstack((centroids, random_centroids))
            centroids, distortion = vq.kmeans(V, centroids, iter=self.nb_iter_kmeans)
            #centroids, distortion = vq.kmeans(V, self.k1, iter=self.nb_iter_kmeans)
            clusters, distortion = vq.vq(V, centroids)

            inds = numpy.argsort(subgraphIndices)
            clustersList.append(clusters[inds.tolist()])

        return clustersList

    def clusterFromIterator(self, graphListIterator, approx=True, timeIter=False, T=10):
        """
        Find a set of clusters for the graphs given by the iterator. If approx
        is True then we use the approximate eigen-update, otherwise we compute
        the whole eigen decomposition. If timeIter is true the each iteration is
        timed and the results are returned as a list.
        
        The difference between a weight matrix and the previous one should be
        positive.
        """
        clustersList = []
        timeList = [] 
        i = 0
        if approx:
            eigenUpdaterWrapper = EigenUpdaterWrapper(self.k2)

        for subW in graphListIterator:
            logging.debug("Clustering graph of size " + str(subW.shape))
            startTime = time.time()
            ABBA = self.shiftLaplacian(subW)

            # --- Eigen value decomposition ---
            if approx and i % T != 0:
                omega, Q = eigenUpdaterWrapper.approxUpdateEig(subW, ABBA, omega, Q)
            else:
                if approx and i != 0:
                    logging.info("Recomputing eigenvectors")
                omega, Q = numpy.linalg.eig(ABBA)

            if approx:
                eigenUpdaterWrapper.storeInformation(subW, ABBA)


            # --- Kmeans ---
            inds = numpy.flipud(numpy.argsort(omega))

            standardiser = Standardiser()
            #For some very strange reason we get an overflow when computing the
            #norm of the rows of Q even though its elements are bounded by 1.
            #We'll ignore it for now
            try:
                V = standardiser.normaliseArray(Q[:, inds[0:self.k1]].real.T).T
            except FloatingPointError as e:
                print("FloatingPointError: " + str(e))
            V = vq.whiten(V)
            if i == 0:
                centroids, distortion = vq.kmeans(V, self.k1, iter=self.nb_iter_kmeans)
            else:
                centroids = self.findCentroids(V, clusters)
                if centroids.shape[0] < self.k1:
                    nb_missing_centroids = self.k1 - centroids.shape[0]
                    random_centroids = V[numpy.random.randint(0, V.shape[0], nb_missing_centroids),:]
                    centroids = numpy.vstack((centroids, random_centroids))
                centroids, distortion = vq.kmeans(V, centroids) #iter can only be 1
            clusters, distortion = vq.vq(V, centroids)

            clustersList.append(clusters)
            timeList.append(time.time()-startTime)

            i += 1

        if timeIter:
            return clustersList, timeList
        else:
            return clustersList

class EigenUpdaterWrapper:
    """
    Wrapper aroud EigenUpdater which stores from one iteration to the other the
    information used by EigenUpdater.
    """
    
    def __init__(self, k):
        self.k = k # number of eigenvectors to keep

    def approxUpdateEig(self, subW, ABBA, omega, Q):
        """
        Update the eigenvalue decomposition of subW.
        """
        deltaDegrees = numpy.sum(subW, 0)[0:self.n] - self.degrees
        inds = numpy.arange(self.n)[deltaDegrees!=0]

        # --- update already existing nodes ---
        Y1 = ABBA[0:self.n, inds] - self.ABBALast[:, inds]
        Y1[inds, :] = Y1[inds, :]/2
        Y2 = numpy.zeros((self.n, inds.shape[0]))
        Y2[(inds, numpy.arange(inds.shape[0]))] = 1
        if Y1.shape[1] != 0:
            omega, Q = EigenUpdater.eigenAdd2(omega, Q, Y1, Y2, self.k)

        # --- add rows/columns ---
        if self.n != ABBA.shape[0]:
            AB = ABBA[0:self.n, self.n:]
            BB = ABBA[self.n:, self.n:]
#            omega, Q = EigenUpdater.eigenConcat(omega, Q, AB, BB, self.k)
            omega, Q = EigenUpdater.lazyEigenConcatAsUpdate(omega, Q, AB, BB, self.k)
        
        return omega, Q
  
    def storeInformation(self, subW, ABBA):
        self.ABBALast = ABBA.copy()
        self.degrees = numpy.sum(subW, 0)
        self.n = ABBA.shape[0]


class IncreasingSubgraphListIterator(object):
    def __init__(self, graph, subgraphIndicesList):
        """
        the $i$-th returned graph is the subgraph of $graph$ using only indices
        $subgraphIndicesList[i]$. Each element of subgraphIndicesList must be a
        subset of the next.
        """
        # args
        self.W = graph.getWeightMatrix()
        self.subgraphIndicesList = subgraphIndicesList
        
        # init variables        
        self.i = 0
        self.subgraphIndices = numpy.array([], numpy.int)

    def __iter__(self):
         return self
         
    def next(self):
        if self.i >= len(self.subgraphIndicesList):
            raise StopIteration
        if self.i == 0:
            self.subgraphIndices = numpy.array(self.subgraphIndicesList[0], numpy.int)
            self.subgraphIndices.sort()
        else:
            newSubgraphIndices = numpy.setdiff1d(numpy.array(self.subgraphIndicesList[self.i], numpy.int), self.subgraphIndices)
            self.subgraphIndices = numpy.r_[self.subgraphIndices, newSubgraphIndices]
        subW = self.W[:, self.subgraphIndices][self.subgraphIndices, :]
        self.i+=1
        return subW
    
class MyDictionary(object):
    def __init__(self):
        self.d = dict()
        self.next_i = 0
            
    def is_known(self, key):
        return key in self.d
    
    def index(self, key):
        if not self.is_known(key):
            self.d[key] = self.next_i
            self.next_i +=1
        return self.d.get(key)

    def __len__(self):
        return len(self.d)

            
class DatedPurchasesGraphListIterator(object):
    def __init__(self, purchasesByWeek):
        """
        The background graph is a bi-partite graph of purchases. Purchases are
        grouped by date (week by week), and we consider the graph of purchases
        between first week and $i$-th week.
        The returned graph considers only users and counts the number of common
        purchases between two users.
        Purchases are given in a list of [user, prod, week, year] with increasing
        date.
        """
        # args
        self.purchasesByWeek = purchasesByWeek

        # init variables        
        self.i_purchase = 0
        self.dictUser = MyDictionary()
        self.dictProd = MyDictionary()
        for user, prod, week, year in purchasesByWeek:
            self.dictUser.index(user)
            self.dictProd.index(prod)
        self.backgroundGraph = DictGraph(False) # directed
        self.W = numpy.zeros((len(self.dictUser), len(self.dictUser)), dtype='int16')
        
    def __iter__(self):
         return self
         
    def next(self):
        if self.i_purchase >= len(self.purchasesByWeek):
            raise StopIteration
        # to check that the week contains new purchases
        W_has_changed = False

        # current date : we skip weeks without purchases
        date = self.purchasesByWeek[self.i_purchase][2:]

        # update graphs adding current-week purchases
        while self.i_purchase<len(self.purchasesByWeek) and self.purchasesByWeek[self.i_purchase][2:] == date:
            user = self.dictUser.index(self.purchasesByWeek[self.i_purchase][0])
            prod = self.dictProd.index(self.purchasesByWeek[self.i_purchase][1])
            # update only if this purchase is seen for the first time
            try:
                newEdge = not self.backgroundGraph.getEdge(prod, user)
            except ValueError:
                newEdge = True
            if newEdge:
                self.backgroundGraph.addEdge(prod, user)
                newCommonPurchases = self.backgroundGraph.neighbours(prod)
#                print prod, newCommonPurchases
                for neighbour in itertools.ifilter(lambda neighbour: neighbour != user, newCommonPurchases):
                    W_has_changed = True
                    self.W[neighbour, user] += 1
                    self.W[user, neighbour] += 1
            self.i_purchase += 1
        if W_has_changed:
          return self.W
        else:
          return self.next()
    
    
    
