"""
An implementation of the incremental spectral clustering method of Ning et al. given in
"Incremental Spectral Clustering by Efficiently Updating the Eigen-system".
"""
import time 
import numpy
import math
import logging
import scipy.linalg
from apgl.util.Util import Util
import scipy.cluster.vq as vq 

class NingSpectralClustering(object):
    def __init__(self, k):
        self.k = k
        self.kmeansIter = 20

    def incrementEigenSystem(self, lmbda, Q, r, W):
        """
        Updated an eigen system with eigenvalues lmbda and eigenvalues Q, with the
        change in incidence vector r. Also we have the weight matrix W. 
        """
        tol = 10**-3

        n = W.shape[0]
        degrees = numpy.sum(W, 0)
        
        #Find nonzero in r then qi and qj, wij, 
        inds = numpy.nonzero(r)[0] 
        deltaW = r[inds[0]]**2 
        deltaW = deltaW.real

        deltaDegrees = numpy.zeros(n)
        deltaDegrees[inds[0]] = deltaW
        deltaDegrees[inds[1]] = deltaW

        deltaL = numpy.zeros((n, n))
        deltaL[inds[0], inds[1]] = -deltaW
        deltaL[inds[1], inds[0]] = -deltaW
        deltaL[inds[0], inds[0]] = deltaW
        deltaL[inds[1], inds[1]] = deltaW

        newLmbda = lmbda.copy()
        newQ = Q.copy() 

        #Assume tau = 0
        largeNeighbours = numpy.union1d(numpy.nonzero(W[:, inds[0]])[0], numpy.nonzero(W[:, inds[1]])[0])
        degreesHat = degrees[largeNeighbours]
        WHat = W[:, largeNeighbours][largeNeighbours, :]
        deltaLHat = deltaL[largeNeighbours, :][:, largeNeighbours]
        QHat = Q[largeNeighbours, :]

        #Estimate each eigenvector in turn
        for s in range(lmbda.shape[0]):
            qi = Q[inds[0], s]
            qj = Q[inds[1], s]

            deltaDeltaQ = tol + 1
            deltaLmbda = 0 
            deltaQ = numpy.zeros(n)

            x = (qi - qj)
            y = (qi**2 + qj**2)
            a = x**2 - lmbda[s]*y
            c = deltaW*y



            iter = 0 

            while deltaDeltaQ > tol and iter < 2 :
                deltaQi = deltaQ[inds[0]]
                deltaQj = deltaQ[inds[1]]

                b = x*(deltaQi - deltaQj) - lmbda[s]*(qi*deltaQi + qj*deltaQj)
                
                d = numpy.sum(QHat[:, s]*degreesHat*deltaQ[largeNeighbours])
                deltaLmbda = deltaW*(a+b)/(1+c+d)

                e = deltaW*(qi*deltaQi + qj*deltaQj)
                print(a, b, c, d)
                print(e)

                #Get a problem when there are no neighbours 
                K = -WHat
                K[numpy.diag_indices(K.shape[0])] += (1-lmbda[s])*degreesHat
                h = (deltaLmbda*degreesHat + lmbda[s]*deltaDegrees[largeNeighbours])*QHat[:, s]
                h -= deltaLHat.dot(QHat[:, s])
                
                lastDeltaQ = deltaQ

                #Note that K can be singular 
                #In the case than the vertices have no edges, we get an error here, and
                #therefore just make no change.
                deltaQ = numpy.zeros(n)
                if K.size != 0:
                    if numpy.isnan(WHat).any():
                        logging.warn("WHat has nan entries")
                    if numpy.isnan(degreesHat).any():
                        logging.warn("degreesHat has nan entries")
                    if numpy.isnan(K).any():

                        logging.warn("K has nan entries")

                    print(lmbda[s])

                    #Fix for weird error in pinv not converging 
                    alpha = 10**-2
                    deltaQ[largeNeighbours] = numpy.linalg.pinv(K.T.dot(K)+ alpha*numpy.eye(K.shape[0]), rcond=10**-20).dot(K.T).dot(h)

                #Compute change in same way as paper? 
                #deltaDeltaQ = numpy.linalg.norm(deltaQ - lastDeltaQ)
                iter += 1 

            newLmbda[s] += deltaLmbda
            newQ[:, s] += deltaQ

        return newLmbda, newQ

    def __updateEigenSystem(self, lmbda, Q, deltaW, W):
        """
        Give the eigenvalues lmbda, eigenvectors Q and a deltaW matrix of weight
        changes, compute sequence of incidence vectors and update eigensystem.
        The deltaW is the change in edges from the current weight martrix which
        is given by W. 
        """
        n = deltaW.shape[0]
        lowTriInds = numpy.tril_indices(n)
        deltaW[lowTriInds] = 0
        changeInds = numpy.nonzero(deltaW)

        R = numpy.zeros((n, changeInds[0].shape[0]))

        for s in range(changeInds[0].shape[0]):
            Util.printIteration(s, 10, changeInds[0].shape[0])
            i = changeInds[0][s]
            j = changeInds[1][s]

            assert deltaW[i, j] >= 0
            print("deltaW=" +str(deltaW[i, j])) 

            sqDeltaW = math.sqrt(deltaW[i, j])
            R[i, s] = sqDeltaW
            R[j, s] = -sqDeltaW
            #Note: update W at each iteration here
            lmbda, Q = self.incrementEigenSystem(lmbda, Q, R[:, s], W)
            W[i, j] += deltaW[i, j]
            W[j, i] += deltaW[i, j]
        
        return lmbda, Q 

    def cluster(self, graphIterator, T=10, timeIter=False):
        """
        Find a set of clusters using the graph and list of subgraph indices. The
        T parameter is how often one recomputes the eigenvalues.
        """
        tol = 10**-6 
        clustersList = []
        timeList = []

        iter = 0 

        for W in graphIterator:
            startTime = time.time()
            logging.info("Graph index:" + str(iter))

            if iter % T != 0:
                #Figure out the similarity changes in existing edges
                n = lastW.shape[0]
                deltaW = W.copy()
                deltaW[0:n, 0:n] = deltaW[0:n, 0:n] - lastW
                
                #If there are vertices added, add zero rows/cols to W with 1 at diag
                WHat = lastW.copy()
                WHat = numpy.c_[numpy.r_[WHat, numpy.zeros((W.shape[0]-n, n))], numpy.zeros((W.shape[1], W.shape[0]-n))]
                WHat[n:, n:] = numpy.eye(W.shape[0]-n)
                
                Q = numpy.r_[Q, numpy.zeros((W.shape[0]-Q.shape[0], Q.shape[1]))]
                lmbda, Q = self.__updateEigenSystem(lmbda, Q, deltaW, WHat)
            else:
                logging.debug("Recomputing eigensystem")
                D = numpy.diag(numpy.sum(W, 0)) + tol*numpy.eye(W.shape[0])
                L = D - W
                lmbda, Q = scipy.linalg.eig(L, D)
                lmbda = lmbda.real
                Q = Q.real
                inds = numpy.argsort(lmbda)[0:self.k]
                lmbda, Q = Util.indEig(lmbda, Q, inds)

            #Now do actual clustering 
            V = vq.whiten(Q)
            centroids, distortion = vq.kmeans(V, self.k, iter=self.kmeansIter)
            clusters, distortion = vq.vq(V, centroids)
            clustersList.append(clusters)
            timeList.append(time.time()-startTime)

            lastW = W.copy()
            iter += 1

        if timeIter:
            return clustersList, timeList
        else:
            return clustersList
